//----------------------------------------------------------------
// Copyright  Antony Corbett 1995. All Rights Reserved
// Author:	Antony Corbett
//				Compuserve	100277,330
//				INTERNET		antony@corbetteng.co.uk
//				Tel +44 (1926) 856131,  Fax +44 (1926) 856727
// Updated: Sergio G. Olivas
//          Parker Hannifin
//          Converted to ComboBox Class
//          06/10/97
//
// Description:
//		Header file for TExpandableComboBox classes.
//
//
//		TComboBox derivatives that provide disabling of selected
//		list items, horizontal scrolling
//
//----------------------------------------------------------------

#ifndef __COMBOEXP_H
#define __COMBOEXP_H

#ifndef OWL_COMBOBOX_H
	#include <owl\combobox.h>
#endif

#if defined BUILDXCBDLL
  #define XCBEXPORT	__export
#elif defined XCBDLL
  #define XCBEXPORT	__import
#else
  #define XCBEXPORT
#endif


enum eExpAlign {
  eExpAlignDefault,  // Default Left Alignment
  eExpAlignAuto,     // Not Implemented
  eExpAlignLeft,     // Aligns (fixes) left side
  eExpAlignRight,    // Aligns (fixes) right side
  eExpAlignCenter};  // Not Implementd

// nUnitWidth
//  if +X  Fixed
//  if 0  No Change in Size
//  if -1 AutoSize to largest string, but not larger than dialog width
//  if -X Size given is maximum Width allowable, will resize to less



//***************************************
//		TCBItemData
//
//		Used to store enable status of an
//		item in the list. Also retains
//		32-bit item data. A ptr to a
//		TCBItemData object is associated
//		with each item added to the list
//		using SeExpItemData.
//
//***************************************


struct XCBEXPORT TCBItemData
	{
	BOOL enabled_;
	BOOL useSysColor_;
	uint32 data_;
	TColor textColor_;

	TCBItemData() :
		enabled_(TRUE),
		data_(0),
		useSysColor_(TRUE)
			{

			}
	};

//***************************************
//		TExpandableComboBox
//
//		Provides horizontal scrolling
//		and disabling of selected items.
//    Color Highlights
//***************************************

class XCBEXPORT TExpandableComboBox : public TComboBox
	{
	private:
      HWND hwndList;
      TRect RectList;
      int   greatestExtent;
      int   nUnitWidth_;
      int   nItemHeight;
      eExpAlign fTCBAlign_;
      bool  bShowCompleteLine_;
      int   nPasses;
      bool bFirstRedraw;

		BOOL hasHScroll_;
		int numTabs_;				// number of tab stops
		int far *tabs_;			// tab stop values

		typedef TISArrayAsVector<int> SortedIntegers;
		SortedIntegers& textExtents_;

	protected:
		BOOL pseudoSingleSel_;	// forced into single-selection mode?

	public:
		TExpandableComboBox(TWindow* parent, int resourceId,
                        eExpAlign fAlign = eExpAlignLeft,
                        int nUnitWidth = -1,
								TModule* module = 0);

		~TExpandableComboBox();

      void ResizeToFit();
		// operations on individual combo box items...
		int AddString(const char far *str);
		int InsertString(const char far *str, int index);
		int DeleteString(int index);
		int AddDisabledString(const char far *str);
		void Enable(int index, BOOL enable = TRUE);
		int SetItemData(int index, uint32 data);
		int SetSel(int index, BOOL select);
		int SetSelIndex(int index);
		void SetTextColor(int index, TColor color);
		void ResetTextColor(int index);
      int GetStringX (char far *str, int nIndex) const;

		// operations on combobox...
		void ClearList();
		BOOL SetTabStops(int numTabs, int far* tabs, bool bShowCompleteLine = false);
      BOOL MakeSingleSel(BOOL single = TRUE);

		// query individual items...
		uint32 GetItemData(int index) const;
		BOOL GetSel(int index) const;
      int GetSelCount() const;
		BOOL GetEnabled(int index) const;
		TCBItemData *GetItemDataPtr(int index) const;

		// query...
		int GetSelIndex() const;

      // Converstions SGO
      void SetupWindow();
      int UnitsToPixels(int nUnits);
      int PixelsToUnits(int nPixels);

	protected:
      LRESULT DefWindowProc(uint msg, WPARAM wParam, LPARAM lParam);

		DECLARE_RESPONSE_TABLE(TExpandableComboBox);

		void CleanupWindow();

		void ODADrawEntire(DRAWITEMSTRUCT &drawInfo);
		void ODAFocus(DRAWITEMSTRUCT &drawInfo);
		void ODASelect(DRAWITEMSTRUCT &drawInfo);

      void CmKillFocus();
		void CmSelChange();
		void CmSelEndOk();

		virtual void DrawListItem(DRAWITEMSTRUCT &drawInfo, const char far *str);
		virtual int FindTextExtent(const char far *str);
		virtual TColor QueryTextColor(DRAWITEMSTRUCT &drawInfo) const;
		virtual TColor QueryBkColor(DRAWITEMSTRUCT &drawInfo) const;
		void StoreStrExtent(const char far *str);
		virtual void DrawText(DRAWITEMSTRUCT &drawInfo,
										const TRect& rc, const char far *str);
		void UpdateHorizontalExtent();
		BOOL IsMultipleSel() const;

		void EvMeasureItem(UINT ctrlId, MEASUREITEMSTRUCT &measure);

	};




//----------------------------------------------------------------
// Description:
//		Determines if the combo box has the multiple select style.
//
// Returns:
//		TRUE/FALSE
//----------------------------------------------------------------
inline BOOL TExpandableComboBox::IsMultipleSel() const
{
  return (FALSE);
}



#endif	//__TXCB_H
